""" - Type Stubs

Auto-generated stub file for type checking and IDE support.

Endpoint: system/settings
Category: cmdb
"""

from __future__ import annotations

from typing import (
    Any,
    ClassVar,
    Literal,
    TypedDict,
)

from hfortix_fortios.models import (
    FortiObject,
    FortiObjectList,
)


# ================================================================
# TypedDict Payloads
# ================================================================

class SettingsGuidefaultpolicycolumnsItem:
    """Nested item for gui-default-policy-columns field - supports attribute access."""
    name: str


class SettingsPayload(TypedDict, total=False):
    """Payload type for Settings operations."""
    comments: str
    vdom_type: Literal["traffic", "lan-extension", "admin"]
    lan_extension_controller_addr: str
    lan_extension_controller_port: int
    opmode: Literal["nat", "transparent"]
    ngfw_mode: Literal["profile-based", "policy-based"]
    http_external_dest: Literal["fortiweb", "forticache"]
    firewall_session_dirty: Literal["check-all", "check-new", "check-policy-option"]
    manageip: str
    gateway: str
    ip: str
    manageip6: str
    gateway6: str
    ip6: str
    device: str
    bfd: Literal["enable", "disable"]
    bfd_desired_min_tx: int
    bfd_required_min_rx: int
    bfd_detect_mult: int
    bfd_dont_enforce_src_port: Literal["enable", "disable"]
    utf8_spam_tagging: Literal["enable", "disable"]
    wccp_cache_engine: Literal["enable", "disable"]
    vpn_stats_log: str | list[str]
    vpn_stats_period: int
    v4_ecmp_mode: Literal["source-ip-based", "weight-based", "usage-based", "source-dest-ip-based"]
    mac_ttl: int
    fw_session_hairpin: Literal["enable", "disable"]
    prp_trailer_action: Literal["enable", "disable"]
    snat_hairpin_traffic: Literal["enable", "disable"]
    dhcp_proxy: Literal["enable", "disable"]
    dhcp_proxy_interface_select_method: Literal["auto", "sdwan", "specify"]
    dhcp_proxy_interface: str
    dhcp_proxy_vrf_select: int
    dhcp_server_ip: str | list[str]
    dhcp6_server_ip: str | list[str]
    central_nat: Literal["enable", "disable"]
    gui_default_policy_columns: str | list[str] | list[dict[str, Any]] | list[SettingsGuidefaultpolicycolumnsItem]
    lldp_reception: Literal["enable", "disable", "global"]
    lldp_transmission: Literal["enable", "disable", "global"]
    link_down_access: Literal["enable", "disable"]
    nat46_generate_ipv6_fragment_header: Literal["enable", "disable"]
    nat46_force_ipv4_packet_forwarding: Literal["enable", "disable"]
    nat64_force_ipv6_packet_forwarding: Literal["enable", "disable"]
    detect_unknown_esp: Literal["enable", "disable"]
    intree_ses_best_route: Literal["force", "disable"]
    auxiliary_session: Literal["enable", "disable"]
    asymroute: Literal["enable", "disable"]
    asymroute_icmp: Literal["enable", "disable"]
    tcp_session_without_syn: Literal["enable", "disable"]
    ses_denied_traffic: Literal["enable", "disable"]
    ses_denied_multicast_traffic: Literal["enable", "disable"]
    strict_src_check: Literal["enable", "disable"]
    allow_linkdown_path: Literal["enable", "disable"]
    asymroute6: Literal["enable", "disable"]
    asymroute6_icmp: Literal["enable", "disable"]
    sctp_session_without_init: Literal["enable", "disable"]
    sip_expectation: Literal["enable", "disable"]
    sip_nat_trace: Literal["enable", "disable"]
    h323_direct_model: Literal["disable", "enable"]
    status: Literal["enable", "disable"]
    sip_tcp_port: int | list[int]
    sip_udp_port: int | list[int]
    sip_ssl_port: int
    sccp_port: int
    multicast_forward: Literal["enable", "disable"]
    multicast_ttl_notchange: Literal["enable", "disable"]
    multicast_skip_policy: Literal["enable", "disable"]
    allow_subnet_overlap: Literal["enable", "disable"]
    deny_tcp_with_icmp: Literal["enable", "disable"]
    ecmp_max_paths: int
    discovered_device_timeout: int
    email_portal_check_dns: Literal["disable", "enable"]
    default_voip_alg_mode: Literal["proxy-based", "kernel-helper-based"]
    gui_icap: Literal["enable", "disable"]
    gui_implicit_policy: Literal["enable", "disable"]
    gui_dns_database: Literal["enable", "disable"]
    gui_load_balance: Literal["enable", "disable"]
    gui_multicast_policy: Literal["enable", "disable"]
    gui_dos_policy: Literal["enable", "disable"]
    gui_object_colors: Literal["enable", "disable"]
    gui_route_tag_address_creation: Literal["enable", "disable"]
    gui_voip_profile: Literal["enable", "disable"]
    gui_ap_profile: Literal["enable", "disable"]
    gui_security_profile_group: Literal["enable", "disable"]
    gui_local_in_policy: Literal["enable", "disable"]
    gui_wanopt_cache: Literal["enable", "disable"]
    gui_explicit_proxy: Literal["enable", "disable"]
    gui_dynamic_routing: Literal["enable", "disable"]
    gui_sslvpn_personal_bookmarks: Literal["enable", "disable"]
    gui_sslvpn_realms: Literal["enable", "disable"]
    gui_policy_based_ipsec: Literal["enable", "disable"]
    gui_threat_weight: Literal["enable", "disable"]
    gui_spamfilter: Literal["enable", "disable"]
    gui_file_filter: Literal["enable", "disable"]
    gui_application_control: Literal["enable", "disable"]
    gui_ips: Literal["enable", "disable"]
    gui_dhcp_advanced: Literal["enable", "disable"]
    gui_vpn: Literal["enable", "disable"]
    gui_sslvpn: Literal["enable", "disable"]
    gui_wireless_controller: Literal["enable", "disable"]
    gui_advanced_wireless_features: Literal["enable", "disable"]
    gui_switch_controller: Literal["enable", "disable"]
    gui_fortiap_split_tunneling: Literal["enable", "disable"]
    gui_webfilter_advanced: Literal["enable", "disable"]
    gui_traffic_shaping: Literal["enable", "disable"]
    gui_wan_load_balancing: Literal["enable", "disable"]
    gui_antivirus: Literal["enable", "disable"]
    gui_webfilter: Literal["enable", "disable"]
    gui_videofilter: Literal["enable", "disable"]
    gui_dnsfilter: Literal["enable", "disable"]
    gui_waf_profile: Literal["enable", "disable"]
    gui_dlp_profile: Literal["enable", "disable"]
    gui_dlp_advanced: Literal["enable", "disable"]
    gui_virtual_patch_profile: Literal["enable", "disable"]
    gui_casb: Literal["enable", "disable"]
    gui_fortiextender_controller: Literal["enable", "disable"]
    gui_advanced_policy: Literal["enable", "disable"]
    gui_allow_unnamed_policy: Literal["enable", "disable"]
    gui_email_collection: Literal["enable", "disable"]
    gui_multiple_interface_policy: Literal["enable", "disable"]
    gui_policy_disclaimer: Literal["enable", "disable"]
    gui_ztna: Literal["enable", "disable"]
    gui_ot: Literal["enable", "disable"]
    gui_dynamic_device_os_id: Literal["enable", "disable"]
    gui_gtp: Literal["enable", "disable"]
    location_id: str
    ike_session_resume: Literal["enable", "disable"]
    ike_quick_crash_detect: Literal["enable", "disable"]
    ike_dn_format: Literal["with-space", "no-space"]
    ike_port: int
    ike_tcp_port: int
    ike_policy_route: Literal["enable", "disable"]
    ike_detailed_event_logs: Literal["disable", "enable"]
    block_land_attack: Literal["disable", "enable"]
    default_app_port_as_service: Literal["enable", "disable"]
    fqdn_session_check: Literal["enable", "disable"]
    ext_resource_session_check: Literal["enable", "disable"]
    dyn_addr_session_check: Literal["enable", "disable"]
    default_policy_expiry_days: int
    gui_enforce_change_summary: Literal["disable", "require", "optional"]
    internet_service_database_cache: Literal["disable", "enable"]
    internet_service_app_ctrl_size: int


# ================================================================
# Response Types (TypedDict for dict-style access)
# ================================================================

class SettingsResponse(TypedDict, total=False):
    """Response type for Settings - use with .dict property for typed dict access."""
    comments: str
    vdom_type: Literal["traffic", "lan-extension", "admin"]
    lan_extension_controller_addr: str
    lan_extension_controller_port: int
    opmode: Literal["nat", "transparent"]
    ngfw_mode: Literal["profile-based", "policy-based"]
    http_external_dest: Literal["fortiweb", "forticache"]
    firewall_session_dirty: Literal["check-all", "check-new", "check-policy-option"]
    manageip: str
    gateway: str
    ip: str
    manageip6: str
    gateway6: str
    ip6: str
    device: str
    bfd: Literal["enable", "disable"]
    bfd_desired_min_tx: int
    bfd_required_min_rx: int
    bfd_detect_mult: int
    bfd_dont_enforce_src_port: Literal["enable", "disable"]
    utf8_spam_tagging: Literal["enable", "disable"]
    wccp_cache_engine: Literal["enable", "disable"]
    vpn_stats_log: str
    vpn_stats_period: int
    v4_ecmp_mode: Literal["source-ip-based", "weight-based", "usage-based", "source-dest-ip-based"]
    mac_ttl: int
    fw_session_hairpin: Literal["enable", "disable"]
    prp_trailer_action: Literal["enable", "disable"]
    snat_hairpin_traffic: Literal["enable", "disable"]
    dhcp_proxy: Literal["enable", "disable"]
    dhcp_proxy_interface_select_method: Literal["auto", "sdwan", "specify"]
    dhcp_proxy_interface: str
    dhcp_proxy_vrf_select: int
    dhcp_server_ip: str | list[str]
    dhcp6_server_ip: str | list[str]
    central_nat: Literal["enable", "disable"]
    gui_default_policy_columns: list[SettingsGuidefaultpolicycolumnsItem]
    lldp_reception: Literal["enable", "disable", "global"]
    lldp_transmission: Literal["enable", "disable", "global"]
    link_down_access: Literal["enable", "disable"]
    nat46_generate_ipv6_fragment_header: Literal["enable", "disable"]
    nat46_force_ipv4_packet_forwarding: Literal["enable", "disable"]
    nat64_force_ipv6_packet_forwarding: Literal["enable", "disable"]
    detect_unknown_esp: Literal["enable", "disable"]
    intree_ses_best_route: Literal["force", "disable"]
    auxiliary_session: Literal["enable", "disable"]
    asymroute: Literal["enable", "disable"]
    asymroute_icmp: Literal["enable", "disable"]
    tcp_session_without_syn: Literal["enable", "disable"]
    ses_denied_traffic: Literal["enable", "disable"]
    ses_denied_multicast_traffic: Literal["enable", "disable"]
    strict_src_check: Literal["enable", "disable"]
    allow_linkdown_path: Literal["enable", "disable"]
    asymroute6: Literal["enable", "disable"]
    asymroute6_icmp: Literal["enable", "disable"]
    sctp_session_without_init: Literal["enable", "disable"]
    sip_expectation: Literal["enable", "disable"]
    sip_nat_trace: Literal["enable", "disable"]
    h323_direct_model: Literal["disable", "enable"]
    status: Literal["enable", "disable"]
    sip_tcp_port: int | list[int]
    sip_udp_port: int | list[int]
    sip_ssl_port: int
    sccp_port: int
    multicast_forward: Literal["enable", "disable"]
    multicast_ttl_notchange: Literal["enable", "disable"]
    multicast_skip_policy: Literal["enable", "disable"]
    allow_subnet_overlap: Literal["enable", "disable"]
    deny_tcp_with_icmp: Literal["enable", "disable"]
    ecmp_max_paths: int
    discovered_device_timeout: int
    email_portal_check_dns: Literal["disable", "enable"]
    default_voip_alg_mode: Literal["proxy-based", "kernel-helper-based"]
    gui_icap: Literal["enable", "disable"]
    gui_implicit_policy: Literal["enable", "disable"]
    gui_dns_database: Literal["enable", "disable"]
    gui_load_balance: Literal["enable", "disable"]
    gui_multicast_policy: Literal["enable", "disable"]
    gui_dos_policy: Literal["enable", "disable"]
    gui_object_colors: Literal["enable", "disable"]
    gui_route_tag_address_creation: Literal["enable", "disable"]
    gui_voip_profile: Literal["enable", "disable"]
    gui_ap_profile: Literal["enable", "disable"]
    gui_security_profile_group: Literal["enable", "disable"]
    gui_local_in_policy: Literal["enable", "disable"]
    gui_wanopt_cache: Literal["enable", "disable"]
    gui_explicit_proxy: Literal["enable", "disable"]
    gui_dynamic_routing: Literal["enable", "disable"]
    gui_sslvpn_personal_bookmarks: Literal["enable", "disable"]
    gui_sslvpn_realms: Literal["enable", "disable"]
    gui_policy_based_ipsec: Literal["enable", "disable"]
    gui_threat_weight: Literal["enable", "disable"]
    gui_spamfilter: Literal["enable", "disable"]
    gui_file_filter: Literal["enable", "disable"]
    gui_application_control: Literal["enable", "disable"]
    gui_ips: Literal["enable", "disable"]
    gui_dhcp_advanced: Literal["enable", "disable"]
    gui_vpn: Literal["enable", "disable"]
    gui_sslvpn: Literal["enable", "disable"]
    gui_wireless_controller: Literal["enable", "disable"]
    gui_advanced_wireless_features: Literal["enable", "disable"]
    gui_switch_controller: Literal["enable", "disable"]
    gui_fortiap_split_tunneling: Literal["enable", "disable"]
    gui_webfilter_advanced: Literal["enable", "disable"]
    gui_traffic_shaping: Literal["enable", "disable"]
    gui_wan_load_balancing: Literal["enable", "disable"]
    gui_antivirus: Literal["enable", "disable"]
    gui_webfilter: Literal["enable", "disable"]
    gui_videofilter: Literal["enable", "disable"]
    gui_dnsfilter: Literal["enable", "disable"]
    gui_waf_profile: Literal["enable", "disable"]
    gui_dlp_profile: Literal["enable", "disable"]
    gui_dlp_advanced: Literal["enable", "disable"]
    gui_virtual_patch_profile: Literal["enable", "disable"]
    gui_casb: Literal["enable", "disable"]
    gui_fortiextender_controller: Literal["enable", "disable"]
    gui_advanced_policy: Literal["enable", "disable"]
    gui_allow_unnamed_policy: Literal["enable", "disable"]
    gui_email_collection: Literal["enable", "disable"]
    gui_multiple_interface_policy: Literal["enable", "disable"]
    gui_policy_disclaimer: Literal["enable", "disable"]
    gui_ztna: Literal["enable", "disable"]
    gui_ot: Literal["enable", "disable"]
    gui_dynamic_device_os_id: Literal["enable", "disable"]
    gui_gtp: Literal["enable", "disable"]
    location_id: str
    ike_session_resume: Literal["enable", "disable"]
    ike_quick_crash_detect: Literal["enable", "disable"]
    ike_dn_format: Literal["with-space", "no-space"]
    ike_port: int
    ike_tcp_port: int
    ike_policy_route: Literal["enable", "disable"]
    ike_detailed_event_logs: Literal["disable", "enable"]
    block_land_attack: Literal["disable", "enable"]
    default_app_port_as_service: Literal["enable", "disable"]
    fqdn_session_check: Literal["enable", "disable"]
    ext_resource_session_check: Literal["enable", "disable"]
    dyn_addr_session_check: Literal["enable", "disable"]
    default_policy_expiry_days: int
    gui_enforce_change_summary: Literal["disable", "require", "optional"]
    internet_service_database_cache: Literal["disable", "enable"]
    internet_service_app_ctrl_size: int


# ================================================================
# Response Types (Class for attribute access)
# ================================================================


class SettingsObject(FortiObject):
    """Typed FortiObject for Settings with field access."""
    comments: str
    vdom_type: Literal["traffic", "lan-extension", "admin"]
    lan_extension_controller_addr: str
    lan_extension_controller_port: int
    opmode: Literal["nat", "transparent"]
    ngfw_mode: Literal["profile-based", "policy-based"]
    http_external_dest: Literal["fortiweb", "forticache"]
    firewall_session_dirty: Literal["check-all", "check-new", "check-policy-option"]
    manageip: str
    gateway: str
    ip: str
    manageip6: str
    gateway6: str
    ip6: str
    device: str
    bfd: Literal["enable", "disable"]
    bfd_desired_min_tx: int
    bfd_required_min_rx: int
    bfd_detect_mult: int
    bfd_dont_enforce_src_port: Literal["enable", "disable"]
    utf8_spam_tagging: Literal["enable", "disable"]
    wccp_cache_engine: Literal["enable", "disable"]
    vpn_stats_log: str
    vpn_stats_period: int
    v4_ecmp_mode: Literal["source-ip-based", "weight-based", "usage-based", "source-dest-ip-based"]
    mac_ttl: int
    fw_session_hairpin: Literal["enable", "disable"]
    prp_trailer_action: Literal["enable", "disable"]
    snat_hairpin_traffic: Literal["enable", "disable"]
    dhcp_proxy: Literal["enable", "disable"]
    dhcp_proxy_interface_select_method: Literal["auto", "sdwan", "specify"]
    dhcp_proxy_interface: str
    dhcp_proxy_vrf_select: int
    dhcp_server_ip: str | list[str]
    dhcp6_server_ip: str | list[str]
    central_nat: Literal["enable", "disable"]
    gui_default_policy_columns: list[SettingsGuidefaultpolicycolumnsItem]
    lldp_reception: Literal["enable", "disable", "global"]
    lldp_transmission: Literal["enable", "disable", "global"]
    link_down_access: Literal["enable", "disable"]
    nat46_generate_ipv6_fragment_header: Literal["enable", "disable"]
    nat46_force_ipv4_packet_forwarding: Literal["enable", "disable"]
    nat64_force_ipv6_packet_forwarding: Literal["enable", "disable"]
    detect_unknown_esp: Literal["enable", "disable"]
    intree_ses_best_route: Literal["force", "disable"]
    auxiliary_session: Literal["enable", "disable"]
    asymroute: Literal["enable", "disable"]
    asymroute_icmp: Literal["enable", "disable"]
    tcp_session_without_syn: Literal["enable", "disable"]
    ses_denied_traffic: Literal["enable", "disable"]
    ses_denied_multicast_traffic: Literal["enable", "disable"]
    strict_src_check: Literal["enable", "disable"]
    allow_linkdown_path: Literal["enable", "disable"]
    asymroute6: Literal["enable", "disable"]
    asymroute6_icmp: Literal["enable", "disable"]
    sctp_session_without_init: Literal["enable", "disable"]
    sip_expectation: Literal["enable", "disable"]
    sip_nat_trace: Literal["enable", "disable"]
    h323_direct_model: Literal["disable", "enable"]
    status: Literal["enable", "disable"]
    sip_tcp_port: int | list[int]
    sip_udp_port: int | list[int]
    sip_ssl_port: int
    sccp_port: int
    multicast_forward: Literal["enable", "disable"]
    multicast_ttl_notchange: Literal["enable", "disable"]
    multicast_skip_policy: Literal["enable", "disable"]
    allow_subnet_overlap: Literal["enable", "disable"]
    deny_tcp_with_icmp: Literal["enable", "disable"]
    ecmp_max_paths: int
    discovered_device_timeout: int
    email_portal_check_dns: Literal["disable", "enable"]
    default_voip_alg_mode: Literal["proxy-based", "kernel-helper-based"]
    gui_icap: Literal["enable", "disable"]
    gui_implicit_policy: Literal["enable", "disable"]
    gui_dns_database: Literal["enable", "disable"]
    gui_load_balance: Literal["enable", "disable"]
    gui_multicast_policy: Literal["enable", "disable"]
    gui_dos_policy: Literal["enable", "disable"]
    gui_object_colors: Literal["enable", "disable"]
    gui_route_tag_address_creation: Literal["enable", "disable"]
    gui_voip_profile: Literal["enable", "disable"]
    gui_ap_profile: Literal["enable", "disable"]
    gui_security_profile_group: Literal["enable", "disable"]
    gui_local_in_policy: Literal["enable", "disable"]
    gui_wanopt_cache: Literal["enable", "disable"]
    gui_explicit_proxy: Literal["enable", "disable"]
    gui_dynamic_routing: Literal["enable", "disable"]
    gui_sslvpn_personal_bookmarks: Literal["enable", "disable"]
    gui_sslvpn_realms: Literal["enable", "disable"]
    gui_policy_based_ipsec: Literal["enable", "disable"]
    gui_threat_weight: Literal["enable", "disable"]
    gui_spamfilter: Literal["enable", "disable"]
    gui_file_filter: Literal["enable", "disable"]
    gui_application_control: Literal["enable", "disable"]
    gui_ips: Literal["enable", "disable"]
    gui_dhcp_advanced: Literal["enable", "disable"]
    gui_vpn: Literal["enable", "disable"]
    gui_sslvpn: Literal["enable", "disable"]
    gui_wireless_controller: Literal["enable", "disable"]
    gui_advanced_wireless_features: Literal["enable", "disable"]
    gui_switch_controller: Literal["enable", "disable"]
    gui_fortiap_split_tunneling: Literal["enable", "disable"]
    gui_webfilter_advanced: Literal["enable", "disable"]
    gui_traffic_shaping: Literal["enable", "disable"]
    gui_wan_load_balancing: Literal["enable", "disable"]
    gui_antivirus: Literal["enable", "disable"]
    gui_webfilter: Literal["enable", "disable"]
    gui_videofilter: Literal["enable", "disable"]
    gui_dnsfilter: Literal["enable", "disable"]
    gui_waf_profile: Literal["enable", "disable"]
    gui_dlp_profile: Literal["enable", "disable"]
    gui_dlp_advanced: Literal["enable", "disable"]
    gui_virtual_patch_profile: Literal["enable", "disable"]
    gui_casb: Literal["enable", "disable"]
    gui_fortiextender_controller: Literal["enable", "disable"]
    gui_advanced_policy: Literal["enable", "disable"]
    gui_allow_unnamed_policy: Literal["enable", "disable"]
    gui_email_collection: Literal["enable", "disable"]
    gui_multiple_interface_policy: Literal["enable", "disable"]
    gui_policy_disclaimer: Literal["enable", "disable"]
    gui_ztna: Literal["enable", "disable"]
    gui_ot: Literal["enable", "disable"]
    gui_dynamic_device_os_id: Literal["enable", "disable"]
    gui_gtp: Literal["enable", "disable"]
    location_id: str
    ike_session_resume: Literal["enable", "disable"]
    ike_quick_crash_detect: Literal["enable", "disable"]
    ike_dn_format: Literal["with-space", "no-space"]
    ike_port: int
    ike_tcp_port: int
    ike_policy_route: Literal["enable", "disable"]
    ike_detailed_event_logs: Literal["disable", "enable"]
    block_land_attack: Literal["disable", "enable"]
    default_app_port_as_service: Literal["enable", "disable"]
    fqdn_session_check: Literal["enable", "disable"]
    ext_resource_session_check: Literal["enable", "disable"]
    dyn_addr_session_check: Literal["enable", "disable"]
    default_policy_expiry_days: int
    gui_enforce_change_summary: Literal["disable", "require", "optional"]
    internet_service_database_cache: Literal["disable", "enable"]
    internet_service_app_ctrl_size: int


# ================================================================
# Main Endpoint Class
# ================================================================

class Settings:
    """
    
    Endpoint: system/settings
    Category: cmdb
    """
    
    # Class attributes for introspection
    endpoint: ClassVar[str] = ...
    path: ClassVar[str] = ...
    category: ClassVar[str] = ...
    capabilities: ClassVar[dict[str, Any]] = ...
    
    def __init__(self, client: Any) -> None:
        """Initialize endpoint with HTTP client."""
        ...
    
    # ================================================================
    # GET Methods
    # ================================================================
    
    # Singleton endpoint (no mkey)
    def get(
        self,
        *,
        filter: str | list[str] | None = ...,
        count: int | None = ...,
        start: int | None = ...,
        payload_dict: dict[str, Any] | None = ...,
        range: list[int] | None = ...,
        sort: str | None = ...,
        format: str | None = ...,
        action: str | None = ...,
        vdom: str | bool | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> SettingsObject: ...
    
    def get_schema(
        self,
        vdom: str | None = ...,
        format: str = ...,
    ) -> FortiObject: ...


    # ================================================================
    # PUT Method
    # ================================================================
    
    def put(
        self,
        payload_dict: SettingsPayload | None = ...,
        comments: str | None = ...,
        vdom_type: Literal["traffic", "lan-extension", "admin"] | None = ...,
        lan_extension_controller_addr: str | None = ...,
        lan_extension_controller_port: int | None = ...,
        opmode: Literal["nat", "transparent"] | None = ...,
        ngfw_mode: Literal["profile-based", "policy-based"] | None = ...,
        http_external_dest: Literal["fortiweb", "forticache"] | None = ...,
        firewall_session_dirty: Literal["check-all", "check-new", "check-policy-option"] | None = ...,
        manageip: str | None = ...,
        gateway: str | None = ...,
        ip: str | None = ...,
        manageip6: str | None = ...,
        gateway6: str | None = ...,
        ip6: str | None = ...,
        device: str | None = ...,
        bfd: Literal["enable", "disable"] | None = ...,
        bfd_desired_min_tx: int | None = ...,
        bfd_required_min_rx: int | None = ...,
        bfd_detect_mult: int | None = ...,
        bfd_dont_enforce_src_port: Literal["enable", "disable"] | None = ...,
        utf8_spam_tagging: Literal["enable", "disable"] | None = ...,
        wccp_cache_engine: Literal["enable", "disable"] | None = ...,
        vpn_stats_log: str | list[str] | None = ...,
        vpn_stats_period: int | None = ...,
        v4_ecmp_mode: Literal["source-ip-based", "weight-based", "usage-based", "source-dest-ip-based"] | None = ...,
        mac_ttl: int | None = ...,
        fw_session_hairpin: Literal["enable", "disable"] | None = ...,
        prp_trailer_action: Literal["enable", "disable"] | None = ...,
        snat_hairpin_traffic: Literal["enable", "disable"] | None = ...,
        dhcp_proxy: Literal["enable", "disable"] | None = ...,
        dhcp_proxy_interface_select_method: Literal["auto", "sdwan", "specify"] | None = ...,
        dhcp_proxy_interface: str | None = ...,
        dhcp_proxy_vrf_select: int | None = ...,
        dhcp_server_ip: str | list[str] | None = ...,
        dhcp6_server_ip: str | list[str] | None = ...,
        central_nat: Literal["enable", "disable"] | None = ...,
        gui_default_policy_columns: str | list[str] | list[dict[str, Any]] | list[SettingsGuidefaultpolicycolumnsItem] | None = ...,
        lldp_reception: Literal["enable", "disable", "global"] | None = ...,
        lldp_transmission: Literal["enable", "disable", "global"] | None = ...,
        link_down_access: Literal["enable", "disable"] | None = ...,
        nat46_generate_ipv6_fragment_header: Literal["enable", "disable"] | None = ...,
        nat46_force_ipv4_packet_forwarding: Literal["enable", "disable"] | None = ...,
        nat64_force_ipv6_packet_forwarding: Literal["enable", "disable"] | None = ...,
        detect_unknown_esp: Literal["enable", "disable"] | None = ...,
        intree_ses_best_route: Literal["force", "disable"] | None = ...,
        auxiliary_session: Literal["enable", "disable"] | None = ...,
        asymroute: Literal["enable", "disable"] | None = ...,
        asymroute_icmp: Literal["enable", "disable"] | None = ...,
        tcp_session_without_syn: Literal["enable", "disable"] | None = ...,
        ses_denied_traffic: Literal["enable", "disable"] | None = ...,
        ses_denied_multicast_traffic: Literal["enable", "disable"] | None = ...,
        strict_src_check: Literal["enable", "disable"] | None = ...,
        allow_linkdown_path: Literal["enable", "disable"] | None = ...,
        asymroute6: Literal["enable", "disable"] | None = ...,
        asymroute6_icmp: Literal["enable", "disable"] | None = ...,
        sctp_session_without_init: Literal["enable", "disable"] | None = ...,
        sip_expectation: Literal["enable", "disable"] | None = ...,
        sip_nat_trace: Literal["enable", "disable"] | None = ...,
        h323_direct_model: Literal["disable", "enable"] | None = ...,
        status: Literal["enable", "disable"] | None = ...,
        sip_tcp_port: int | list[int] | None = ...,
        sip_udp_port: int | list[int] | None = ...,
        sip_ssl_port: int | None = ...,
        sccp_port: int | None = ...,
        multicast_forward: Literal["enable", "disable"] | None = ...,
        multicast_ttl_notchange: Literal["enable", "disable"] | None = ...,
        multicast_skip_policy: Literal["enable", "disable"] | None = ...,
        allow_subnet_overlap: Literal["enable", "disable"] | None = ...,
        deny_tcp_with_icmp: Literal["enable", "disable"] | None = ...,
        ecmp_max_paths: int | None = ...,
        discovered_device_timeout: int | None = ...,
        email_portal_check_dns: Literal["disable", "enable"] | None = ...,
        default_voip_alg_mode: Literal["proxy-based", "kernel-helper-based"] | None = ...,
        gui_icap: Literal["enable", "disable"] | None = ...,
        gui_implicit_policy: Literal["enable", "disable"] | None = ...,
        gui_dns_database: Literal["enable", "disable"] | None = ...,
        gui_load_balance: Literal["enable", "disable"] | None = ...,
        gui_multicast_policy: Literal["enable", "disable"] | None = ...,
        gui_dos_policy: Literal["enable", "disable"] | None = ...,
        gui_object_colors: Literal["enable", "disable"] | None = ...,
        gui_route_tag_address_creation: Literal["enable", "disable"] | None = ...,
        gui_voip_profile: Literal["enable", "disable"] | None = ...,
        gui_ap_profile: Literal["enable", "disable"] | None = ...,
        gui_security_profile_group: Literal["enable", "disable"] | None = ...,
        gui_local_in_policy: Literal["enable", "disable"] | None = ...,
        gui_wanopt_cache: Literal["enable", "disable"] | None = ...,
        gui_explicit_proxy: Literal["enable", "disable"] | None = ...,
        gui_dynamic_routing: Literal["enable", "disable"] | None = ...,
        gui_sslvpn_personal_bookmarks: Literal["enable", "disable"] | None = ...,
        gui_sslvpn_realms: Literal["enable", "disable"] | None = ...,
        gui_policy_based_ipsec: Literal["enable", "disable"] | None = ...,
        gui_threat_weight: Literal["enable", "disable"] | None = ...,
        gui_spamfilter: Literal["enable", "disable"] | None = ...,
        gui_file_filter: Literal["enable", "disable"] | None = ...,
        gui_application_control: Literal["enable", "disable"] | None = ...,
        gui_ips: Literal["enable", "disable"] | None = ...,
        gui_dhcp_advanced: Literal["enable", "disable"] | None = ...,
        gui_vpn: Literal["enable", "disable"] | None = ...,
        gui_sslvpn: Literal["enable", "disable"] | None = ...,
        gui_wireless_controller: Literal["enable", "disable"] | None = ...,
        gui_advanced_wireless_features: Literal["enable", "disable"] | None = ...,
        gui_switch_controller: Literal["enable", "disable"] | None = ...,
        gui_fortiap_split_tunneling: Literal["enable", "disable"] | None = ...,
        gui_webfilter_advanced: Literal["enable", "disable"] | None = ...,
        gui_traffic_shaping: Literal["enable", "disable"] | None = ...,
        gui_wan_load_balancing: Literal["enable", "disable"] | None = ...,
        gui_antivirus: Literal["enable", "disable"] | None = ...,
        gui_webfilter: Literal["enable", "disable"] | None = ...,
        gui_videofilter: Literal["enable", "disable"] | None = ...,
        gui_dnsfilter: Literal["enable", "disable"] | None = ...,
        gui_waf_profile: Literal["enable", "disable"] | None = ...,
        gui_dlp_profile: Literal["enable", "disable"] | None = ...,
        gui_dlp_advanced: Literal["enable", "disable"] | None = ...,
        gui_virtual_patch_profile: Literal["enable", "disable"] | None = ...,
        gui_casb: Literal["enable", "disable"] | None = ...,
        gui_fortiextender_controller: Literal["enable", "disable"] | None = ...,
        gui_advanced_policy: Literal["enable", "disable"] | None = ...,
        gui_allow_unnamed_policy: Literal["enable", "disable"] | None = ...,
        gui_email_collection: Literal["enable", "disable"] | None = ...,
        gui_multiple_interface_policy: Literal["enable", "disable"] | None = ...,
        gui_policy_disclaimer: Literal["enable", "disable"] | None = ...,
        gui_ztna: Literal["enable", "disable"] | None = ...,
        gui_ot: Literal["enable", "disable"] | None = ...,
        gui_dynamic_device_os_id: Literal["enable", "disable"] | None = ...,
        gui_gtp: Literal["enable", "disable"] | None = ...,
        location_id: str | None = ...,
        ike_session_resume: Literal["enable", "disable"] | None = ...,
        ike_quick_crash_detect: Literal["enable", "disable"] | None = ...,
        ike_dn_format: Literal["with-space", "no-space"] | None = ...,
        ike_port: int | None = ...,
        ike_tcp_port: int | None = ...,
        ike_policy_route: Literal["enable", "disable"] | None = ...,
        ike_detailed_event_logs: Literal["disable", "enable"] | None = ...,
        block_land_attack: Literal["disable", "enable"] | None = ...,
        default_app_port_as_service: Literal["enable", "disable"] | None = ...,
        fqdn_session_check: Literal["enable", "disable"] | None = ...,
        ext_resource_session_check: Literal["enable", "disable"] | None = ...,
        dyn_addr_session_check: Literal["enable", "disable"] | None = ...,
        default_policy_expiry_days: int | None = ...,
        gui_enforce_change_summary: Literal["disable", "require", "optional"] | None = ...,
        internet_service_database_cache: Literal["disable", "enable"] | None = ...,
        internet_service_app_ctrl_size: int | None = ...,
        vdom: str | bool | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> SettingsObject: ...


    # ================================================================
    # Utility Methods
    # ================================================================
    
    def exists(
        self,
        name: str,
        vdom: str | bool | None = ...,
    ) -> bool: ...
    
    def set(
        self,
        payload_dict: SettingsPayload | None = ...,
        comments: str | None = ...,
        vdom_type: Literal["traffic", "lan-extension", "admin"] | None = ...,
        lan_extension_controller_addr: str | None = ...,
        lan_extension_controller_port: int | None = ...,
        opmode: Literal["nat", "transparent"] | None = ...,
        ngfw_mode: Literal["profile-based", "policy-based"] | None = ...,
        http_external_dest: Literal["fortiweb", "forticache"] | None = ...,
        firewall_session_dirty: Literal["check-all", "check-new", "check-policy-option"] | None = ...,
        manageip: str | None = ...,
        gateway: str | None = ...,
        ip: str | None = ...,
        manageip6: str | None = ...,
        gateway6: str | None = ...,
        ip6: str | None = ...,
        device: str | None = ...,
        bfd: Literal["enable", "disable"] | None = ...,
        bfd_desired_min_tx: int | None = ...,
        bfd_required_min_rx: int | None = ...,
        bfd_detect_mult: int | None = ...,
        bfd_dont_enforce_src_port: Literal["enable", "disable"] | None = ...,
        utf8_spam_tagging: Literal["enable", "disable"] | None = ...,
        wccp_cache_engine: Literal["enable", "disable"] | None = ...,
        vpn_stats_log: Literal["ipsec", "pptp", "l2tp", "ssl"] | list[str] | None = ...,
        vpn_stats_period: int | None = ...,
        v4_ecmp_mode: Literal["source-ip-based", "weight-based", "usage-based", "source-dest-ip-based"] | None = ...,
        mac_ttl: int | None = ...,
        fw_session_hairpin: Literal["enable", "disable"] | None = ...,
        prp_trailer_action: Literal["enable", "disable"] | None = ...,
        snat_hairpin_traffic: Literal["enable", "disable"] | None = ...,
        dhcp_proxy: Literal["enable", "disable"] | None = ...,
        dhcp_proxy_interface_select_method: Literal["auto", "sdwan", "specify"] | None = ...,
        dhcp_proxy_interface: str | None = ...,
        dhcp_proxy_vrf_select: int | None = ...,
        dhcp_server_ip: str | list[str] | None = ...,
        dhcp6_server_ip: str | list[str] | None = ...,
        central_nat: Literal["enable", "disable"] | None = ...,
        gui_default_policy_columns: str | list[str] | list[dict[str, Any]] | list[SettingsGuidefaultpolicycolumnsItem] | None = ...,
        lldp_reception: Literal["enable", "disable", "global"] | None = ...,
        lldp_transmission: Literal["enable", "disable", "global"] | None = ...,
        link_down_access: Literal["enable", "disable"] | None = ...,
        nat46_generate_ipv6_fragment_header: Literal["enable", "disable"] | None = ...,
        nat46_force_ipv4_packet_forwarding: Literal["enable", "disable"] | None = ...,
        nat64_force_ipv6_packet_forwarding: Literal["enable", "disable"] | None = ...,
        detect_unknown_esp: Literal["enable", "disable"] | None = ...,
        intree_ses_best_route: Literal["force", "disable"] | None = ...,
        auxiliary_session: Literal["enable", "disable"] | None = ...,
        asymroute: Literal["enable", "disable"] | None = ...,
        asymroute_icmp: Literal["enable", "disable"] | None = ...,
        tcp_session_without_syn: Literal["enable", "disable"] | None = ...,
        ses_denied_traffic: Literal["enable", "disable"] | None = ...,
        ses_denied_multicast_traffic: Literal["enable", "disable"] | None = ...,
        strict_src_check: Literal["enable", "disable"] | None = ...,
        allow_linkdown_path: Literal["enable", "disable"] | None = ...,
        asymroute6: Literal["enable", "disable"] | None = ...,
        asymroute6_icmp: Literal["enable", "disable"] | None = ...,
        sctp_session_without_init: Literal["enable", "disable"] | None = ...,
        sip_expectation: Literal["enable", "disable"] | None = ...,
        sip_nat_trace: Literal["enable", "disable"] | None = ...,
        h323_direct_model: Literal["disable", "enable"] | None = ...,
        status: Literal["enable", "disable"] | None = ...,
        sip_tcp_port: int | list[int] | None = ...,
        sip_udp_port: int | list[int] | None = ...,
        sip_ssl_port: int | None = ...,
        sccp_port: int | None = ...,
        multicast_forward: Literal["enable", "disable"] | None = ...,
        multicast_ttl_notchange: Literal["enable", "disable"] | None = ...,
        multicast_skip_policy: Literal["enable", "disable"] | None = ...,
        allow_subnet_overlap: Literal["enable", "disable"] | None = ...,
        deny_tcp_with_icmp: Literal["enable", "disable"] | None = ...,
        ecmp_max_paths: int | None = ...,
        discovered_device_timeout: int | None = ...,
        email_portal_check_dns: Literal["disable", "enable"] | None = ...,
        default_voip_alg_mode: Literal["proxy-based", "kernel-helper-based"] | None = ...,
        gui_icap: Literal["enable", "disable"] | None = ...,
        gui_implicit_policy: Literal["enable", "disable"] | None = ...,
        gui_dns_database: Literal["enable", "disable"] | None = ...,
        gui_load_balance: Literal["enable", "disable"] | None = ...,
        gui_multicast_policy: Literal["enable", "disable"] | None = ...,
        gui_dos_policy: Literal["enable", "disable"] | None = ...,
        gui_object_colors: Literal["enable", "disable"] | None = ...,
        gui_route_tag_address_creation: Literal["enable", "disable"] | None = ...,
        gui_voip_profile: Literal["enable", "disable"] | None = ...,
        gui_ap_profile: Literal["enable", "disable"] | None = ...,
        gui_security_profile_group: Literal["enable", "disable"] | None = ...,
        gui_local_in_policy: Literal["enable", "disable"] | None = ...,
        gui_wanopt_cache: Literal["enable", "disable"] | None = ...,
        gui_explicit_proxy: Literal["enable", "disable"] | None = ...,
        gui_dynamic_routing: Literal["enable", "disable"] | None = ...,
        gui_sslvpn_personal_bookmarks: Literal["enable", "disable"] | None = ...,
        gui_sslvpn_realms: Literal["enable", "disable"] | None = ...,
        gui_policy_based_ipsec: Literal["enable", "disable"] | None = ...,
        gui_threat_weight: Literal["enable", "disable"] | None = ...,
        gui_spamfilter: Literal["enable", "disable"] | None = ...,
        gui_file_filter: Literal["enable", "disable"] | None = ...,
        gui_application_control: Literal["enable", "disable"] | None = ...,
        gui_ips: Literal["enable", "disable"] | None = ...,
        gui_dhcp_advanced: Literal["enable", "disable"] | None = ...,
        gui_vpn: Literal["enable", "disable"] | None = ...,
        gui_sslvpn: Literal["enable", "disable"] | None = ...,
        gui_wireless_controller: Literal["enable", "disable"] | None = ...,
        gui_advanced_wireless_features: Literal["enable", "disable"] | None = ...,
        gui_switch_controller: Literal["enable", "disable"] | None = ...,
        gui_fortiap_split_tunneling: Literal["enable", "disable"] | None = ...,
        gui_webfilter_advanced: Literal["enable", "disable"] | None = ...,
        gui_traffic_shaping: Literal["enable", "disable"] | None = ...,
        gui_wan_load_balancing: Literal["enable", "disable"] | None = ...,
        gui_antivirus: Literal["enable", "disable"] | None = ...,
        gui_webfilter: Literal["enable", "disable"] | None = ...,
        gui_videofilter: Literal["enable", "disable"] | None = ...,
        gui_dnsfilter: Literal["enable", "disable"] | None = ...,
        gui_waf_profile: Literal["enable", "disable"] | None = ...,
        gui_dlp_profile: Literal["enable", "disable"] | None = ...,
        gui_dlp_advanced: Literal["enable", "disable"] | None = ...,
        gui_virtual_patch_profile: Literal["enable", "disable"] | None = ...,
        gui_casb: Literal["enable", "disable"] | None = ...,
        gui_fortiextender_controller: Literal["enable", "disable"] | None = ...,
        gui_advanced_policy: Literal["enable", "disable"] | None = ...,
        gui_allow_unnamed_policy: Literal["enable", "disable"] | None = ...,
        gui_email_collection: Literal["enable", "disable"] | None = ...,
        gui_multiple_interface_policy: Literal["enable", "disable"] | None = ...,
        gui_policy_disclaimer: Literal["enable", "disable"] | None = ...,
        gui_ztna: Literal["enable", "disable"] | None = ...,
        gui_ot: Literal["enable", "disable"] | None = ...,
        gui_dynamic_device_os_id: Literal["enable", "disable"] | None = ...,
        gui_gtp: Literal["enable", "disable"] | None = ...,
        location_id: str | None = ...,
        ike_session_resume: Literal["enable", "disable"] | None = ...,
        ike_quick_crash_detect: Literal["enable", "disable"] | None = ...,
        ike_dn_format: Literal["with-space", "no-space"] | None = ...,
        ike_port: int | None = ...,
        ike_tcp_port: int | None = ...,
        ike_policy_route: Literal["enable", "disable"] | None = ...,
        ike_detailed_event_logs: Literal["disable", "enable"] | None = ...,
        block_land_attack: Literal["disable", "enable"] | None = ...,
        default_app_port_as_service: Literal["enable", "disable"] | None = ...,
        fqdn_session_check: Literal["enable", "disable"] | None = ...,
        ext_resource_session_check: Literal["enable", "disable"] | None = ...,
        dyn_addr_session_check: Literal["enable", "disable"] | None = ...,
        default_policy_expiry_days: int | None = ...,
        gui_enforce_change_summary: Literal["disable", "require", "optional"] | None = ...,
        internet_service_database_cache: Literal["disable", "enable"] | None = ...,
        internet_service_app_ctrl_size: int | None = ...,
        vdom: str | bool | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> FortiObject: ...
    
    # Helper methods
    @staticmethod
    def help(field_name: str | None = ...) -> str: ...
    
    @staticmethod
    def fields(detailed: bool = ...) -> list[str] | list[dict[str, Any]]: ...
    
    @staticmethod
    def field_info(field_name: str) -> FortiObject: ...
    
    @staticmethod
    def validate_field(name: str, value: Any) -> bool: ...
    
    @staticmethod
    def required_fields() -> list[str]: ...
    
    @staticmethod
    def defaults() -> FortiObject: ...
    
    @staticmethod
    def schema() -> FortiObject: ...


__all__ = [
    "Settings",
    "SettingsPayload",
    "SettingsResponse",
    "SettingsObject",
]


"""Validation helpers for user/local - Auto-generated"""

from typing import Any, TypedDict, Literal
from typing_extensions import NotRequired

# Import common validators from central _helpers module
from hfortix_fortios._helpers import (
    validate_enable_disable,
    validate_integer_range,
    validate_string_length,
    validate_port_number,
    validate_ip_address,
    validate_ipv6_address,
    validate_mac_address,
)

# Import central validation functions (avoid duplication across 1,062 files)
from hfortix_fortios._helpers.validation import (
    validate_required_fields as _validate_required_fields,
    validate_enum_field as _validate_enum_field,
    validate_query_parameter as _validate_query_parameter,
)

# ============================================================================
# Required Fields Validation
# Auto-generated from schema
# ============================================================================

# ⚠️  IMPORTANT: FortiOS schemas have known issues with required field marking:

# Do NOT use this list for strict validation - test with the actual FortiOS API!

# Fields marked as required (after filtering false positives)
REQUIRED_FIELDS = [
    "passwd",  # User's password.
    "ldap-server",  # Name of LDAP server with which the user must authenticate.
    "radius-server",  # Name of RADIUS server with which the user must authenticate.
    "tacacs+-server",  # Name of TACACS+ server with which the user must authenticate.
    "saml-server",  # Name of SAML server with which the user must authenticate.
]

# Fields with defaults (optional)
FIELDS_WITH_DEFAULTS = {
    "name": "",
    "id": 0,
    "status": "enable",
    "type": "password",
    "ldap-server": "",
    "radius-server": "",
    "tacacs+-server": "",
    "saml-server": "",
    "two-factor": "disable",
    "two-factor-authentication": "",
    "two-factor-notification": "",
    "fortitoken": "",
    "email-to": "",
    "sms-server": "fortiguard",
    "sms-custom-server": "",
    "sms-phone": "",
    "passwd-policy": "",
    "passwd-time": "",
    "authtimeout": 0,
    "workstation": "",
    "auth-concurrent-override": "disable",
    "auth-concurrent-value": 0,
    "ppk-identity": "",
    "qkd-profile": "",
    "username-sensitivity": "enable",
}

# ============================================================================
# Deprecated Fields
# Auto-generated from schema - warns users about deprecated fields
# ============================================================================

# Deprecated fields with migration guidance
DEPRECATED_FIELDS = {
}

# ============================================================================
# Field Metadata (Type Information & Descriptions)
# Auto-generated from schema - use for IDE autocomplete and documentation
# ============================================================================

# Field types mapping
FIELD_TYPES = {
    "name": "string",  # Local user name.
    "id": "integer",  # User ID.
    "status": "option",  # Enable/disable allowing the local user to authenticate with 
    "type": "option",  # Authentication method.
    "passwd": "password",  # User's password.
    "ldap-server": "string",  # Name of LDAP server with which the user must authenticate.
    "radius-server": "string",  # Name of RADIUS server with which the user must authenticate.
    "tacacs+-server": "string",  # Name of TACACS+ server with which the user must authenticate
    "saml-server": "string",  # Name of SAML server with which the user must authenticate.
    "two-factor": "option",  # Enable/disable two-factor authentication.
    "two-factor-authentication": "option",  # Authentication method by FortiToken Cloud.
    "two-factor-notification": "option",  # Notification method for user activation by FortiToken Cloud.
    "fortitoken": "string",  # Two-factor recipient's FortiToken serial number.
    "email-to": "string",  # Two-factor recipient's email address.
    "sms-server": "option",  # Send SMS through FortiGuard or other external server.
    "sms-custom-server": "string",  # Two-factor recipient's SMS server.
    "sms-phone": "string",  # Two-factor recipient's mobile phone number.
    "passwd-policy": "string",  # Password policy to apply to this user, as defined in config 
    "passwd-time": "user",  # Time of the last password update.
    "authtimeout": "integer",  # Time in minutes before the authentication timeout for a user
    "workstation": "string",  # Name of the remote user workstation, if you want to limit th
    "auth-concurrent-override": "option",  # Enable/disable overriding the policy-auth-concurrent under c
    "auth-concurrent-value": "integer",  # Maximum number of concurrent logins permitted from the same 
    "ppk-secret": "password-3",  # IKEv2 Postquantum Preshared Key (ASCII string or hexadecimal
    "ppk-identity": "string",  # IKEv2 Postquantum Preshared Key Identity.
    "qkd-profile": "string",  # Quantum Key Distribution (QKD) profile.
    "username-sensitivity": "option",  # Enable/disable case and accent sensitivity when performing u
}

# Field descriptions (help text from FortiOS API)
FIELD_DESCRIPTIONS = {
    "name": "Local user name.",
    "id": "User ID.",
    "status": "Enable/disable allowing the local user to authenticate with the FortiGate unit.",
    "type": "Authentication method.",
    "passwd": "User's password.",
    "ldap-server": "Name of LDAP server with which the user must authenticate.",
    "radius-server": "Name of RADIUS server with which the user must authenticate.",
    "tacacs+-server": "Name of TACACS+ server with which the user must authenticate.",
    "saml-server": "Name of SAML server with which the user must authenticate.",
    "two-factor": "Enable/disable two-factor authentication.",
    "two-factor-authentication": "Authentication method by FortiToken Cloud.",
    "two-factor-notification": "Notification method for user activation by FortiToken Cloud.",
    "fortitoken": "Two-factor recipient's FortiToken serial number.",
    "email-to": "Two-factor recipient's email address.",
    "sms-server": "Send SMS through FortiGuard or other external server.",
    "sms-custom-server": "Two-factor recipient's SMS server.",
    "sms-phone": "Two-factor recipient's mobile phone number.",
    "passwd-policy": "Password policy to apply to this user, as defined in config user password-policy.",
    "passwd-time": "Time of the last password update.",
    "authtimeout": "Time in minutes before the authentication timeout for a user is reached.",
    "workstation": "Name of the remote user workstation, if you want to limit the user to authenticate only from a particular workstation.",
    "auth-concurrent-override": "Enable/disable overriding the policy-auth-concurrent under config system global.",
    "auth-concurrent-value": "Maximum number of concurrent logins permitted from the same user.",
    "ppk-secret": "IKEv2 Postquantum Preshared Key (ASCII string or hexadecimal encoded with a leading 0x).",
    "ppk-identity": "IKEv2 Postquantum Preshared Key Identity.",
    "qkd-profile": "Quantum Key Distribution (QKD) profile.",
    "username-sensitivity": "Enable/disable case and accent sensitivity when performing username matching (accents are stripped and case is ignored when disabled).",
}

# Field constraints (string lengths, integer ranges)
FIELD_CONSTRAINTS = {
    "name": {"type": "string", "max_length": 64},
    "id": {"type": "integer", "min": 0, "max": 4294967295},
    "ldap-server": {"type": "string", "max_length": 35},
    "radius-server": {"type": "string", "max_length": 35},
    "tacacs+-server": {"type": "string", "max_length": 35},
    "saml-server": {"type": "string", "max_length": 35},
    "fortitoken": {"type": "string", "max_length": 16},
    "email-to": {"type": "string", "max_length": 63},
    "sms-custom-server": {"type": "string", "max_length": 35},
    "sms-phone": {"type": "string", "max_length": 15},
    "passwd-policy": {"type": "string", "max_length": 35},
    "authtimeout": {"type": "integer", "min": 0, "max": 1440},
    "workstation": {"type": "string", "max_length": 35},
    "auth-concurrent-value": {"type": "integer", "min": 0, "max": 100},
    "ppk-identity": {"type": "string", "max_length": 35},
    "qkd-profile": {"type": "string", "max_length": 35},
}

# Nested schemas (for table/list fields with children)
NESTED_SCHEMAS = {
}


# Valid enum values from API documentation
VALID_BODY_STATUS = [
    "enable",
    "disable",
]
VALID_BODY_TYPE = [
    "password",
    "radius",
    "tacacs+",
    "ldap",
    "saml",
]
VALID_BODY_TWO_FACTOR = [
    "disable",
    "fortitoken",
    "fortitoken-cloud",
    "email",
    "sms",
]
VALID_BODY_TWO_FACTOR_AUTHENTICATION = [
    "fortitoken",
    "email",
    "sms",
]
VALID_BODY_TWO_FACTOR_NOTIFICATION = [
    "email",
    "sms",
]
VALID_BODY_SMS_SERVER = [
    "fortiguard",
    "custom",
]
VALID_BODY_AUTH_CONCURRENT_OVERRIDE = [
    "enable",
    "disable",
]
VALID_BODY_USERNAME_SENSITIVITY = [
    "disable",
    "enable",
]
VALID_QUERY_ACTION = ["default", "schema"]

# ============================================================================
# GET Validation
# ============================================================================


def validate_user_local_get(
    attr: str | None = None,
    filters: dict[str, Any] | None = None,
    **params: Any,
) -> tuple[bool, str | None]:
    """Validate GET request parameters for user/local."""
    # Validate query parameters using central function
    if "action" in params:
        is_valid, error = _validate_query_parameter(
            "action",
            params.get("action"),
            VALID_QUERY_ACTION
        )
        if not is_valid:
            return (False, error)

    return (True, None)


# ============================================================================
# POST Validation
# ============================================================================


def validate_user_local_post(
    payload: dict,
    **params: Any,
) -> tuple[bool, str | None]:
    """Validate POST request to create new user/local object."""
    # Step 1: Validate required fields using central function
    is_valid, error = _validate_required_fields(
        payload,
        REQUIRED_FIELDS,
        FIELD_DESCRIPTIONS
    )
    if not is_valid:
        return (False, error)

    # Step 2: Validate enum values using central function
    if "status" in payload:
        is_valid, error = _validate_enum_field(
            "status",
            payload["status"],
            VALID_BODY_STATUS,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "type" in payload:
        is_valid, error = _validate_enum_field(
            "type",
            payload["type"],
            VALID_BODY_TYPE,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "two-factor" in payload:
        is_valid, error = _validate_enum_field(
            "two-factor",
            payload["two-factor"],
            VALID_BODY_TWO_FACTOR,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "two-factor-authentication" in payload:
        is_valid, error = _validate_enum_field(
            "two-factor-authentication",
            payload["two-factor-authentication"],
            VALID_BODY_TWO_FACTOR_AUTHENTICATION,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "two-factor-notification" in payload:
        is_valid, error = _validate_enum_field(
            "two-factor-notification",
            payload["two-factor-notification"],
            VALID_BODY_TWO_FACTOR_NOTIFICATION,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "sms-server" in payload:
        is_valid, error = _validate_enum_field(
            "sms-server",
            payload["sms-server"],
            VALID_BODY_SMS_SERVER,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "auth-concurrent-override" in payload:
        is_valid, error = _validate_enum_field(
            "auth-concurrent-override",
            payload["auth-concurrent-override"],
            VALID_BODY_AUTH_CONCURRENT_OVERRIDE,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "username-sensitivity" in payload:
        is_valid, error = _validate_enum_field(
            "username-sensitivity",
            payload["username-sensitivity"],
            VALID_BODY_USERNAME_SENSITIVITY,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)

    return (True, None)


# ============================================================================
# PUT Validation
# ============================================================================


def validate_user_local_put(
    payload: dict,
    **params: Any,
) -> tuple[bool, str | None]:
    """Validate PUT request to update user/local."""
    # Validate enum values using central function
    if "status" in payload:
        is_valid, error = _validate_enum_field(
            "status",
            payload["status"],
            VALID_BODY_STATUS,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "type" in payload:
        is_valid, error = _validate_enum_field(
            "type",
            payload["type"],
            VALID_BODY_TYPE,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "two-factor" in payload:
        is_valid, error = _validate_enum_field(
            "two-factor",
            payload["two-factor"],
            VALID_BODY_TWO_FACTOR,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "two-factor-authentication" in payload:
        is_valid, error = _validate_enum_field(
            "two-factor-authentication",
            payload["two-factor-authentication"],
            VALID_BODY_TWO_FACTOR_AUTHENTICATION,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "two-factor-notification" in payload:
        is_valid, error = _validate_enum_field(
            "two-factor-notification",
            payload["two-factor-notification"],
            VALID_BODY_TWO_FACTOR_NOTIFICATION,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "sms-server" in payload:
        is_valid, error = _validate_enum_field(
            "sms-server",
            payload["sms-server"],
            VALID_BODY_SMS_SERVER,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "auth-concurrent-override" in payload:
        is_valid, error = _validate_enum_field(
            "auth-concurrent-override",
            payload["auth-concurrent-override"],
            VALID_BODY_AUTH_CONCURRENT_OVERRIDE,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "username-sensitivity" in payload:
        is_valid, error = _validate_enum_field(
            "username-sensitivity",
            payload["username-sensitivity"],
            VALID_BODY_USERNAME_SENSITIVITY,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)

    return (True, None)


# ============================================================================
# Metadata Access Functions
# Imported from central module to avoid duplication across 1,062 files
# Bound to this endpoint's data using functools.partial (saves ~7KB per file)
# ============================================================================

from functools import partial
from hfortix_fortios._helpers.metadata import (
    get_field_description,
    get_field_type,
    get_field_constraints,
    get_field_default,
    get_field_options,
    get_nested_schema,
    get_all_fields,
    get_field_metadata,
    validate_field_value,
)

# Bind module-specific data to central functions using partial application
get_field_description = partial(get_field_description, FIELD_DESCRIPTIONS)
get_field_type = partial(get_field_type, FIELD_TYPES)
get_field_constraints = partial(get_field_constraints, FIELD_CONSTRAINTS)
get_field_default = partial(get_field_default, FIELDS_WITH_DEFAULTS)
get_field_options = partial(get_field_options, globals())
get_nested_schema = partial(get_nested_schema, NESTED_SCHEMAS)
get_all_fields = partial(get_all_fields, FIELD_TYPES)
get_field_metadata = partial(get_field_metadata, FIELD_TYPES, FIELD_DESCRIPTIONS, 
                             FIELD_CONSTRAINTS, FIELDS_WITH_DEFAULTS, REQUIRED_FIELDS,
                             NESTED_SCHEMAS, globals())
validate_field_value = partial(validate_field_value, FIELD_TYPES, FIELD_DESCRIPTIONS,
                               FIELD_CONSTRAINTS, globals())


# ============================================================================
# Schema Information
# Metadata about this endpoint schema
# ============================================================================

SCHEMA_INFO = {
    "endpoint": "user/local",
    "category": "cmdb",
    "api_path": "user/local",
    "mkey": "name",
    "mkey_type": "string",
    "help": "Configure local users.",
    "total_fields": 27,
    "required_fields_count": 5,
    "fields_with_defaults_count": 25,
}


def get_schema_info() -> dict[str, Any]:
    """Get information about this endpoint schema."""
    return SCHEMA_INFO.copy()



""" - Type Stubs

Auto-generated stub file for type checking and IDE support.

Endpoint: user/exchange
Category: cmdb
"""

from __future__ import annotations

from typing import (
    Any,
    ClassVar,
    Literal,
    TypedDict,
    overload,
)

from hfortix_fortios.models import (
    FortiObject,
    FortiObjectList,
)


# ================================================================
# TypedDict Payloads
# ================================================================

class ExchangeKdcipItem:
    """Nested item for kdc-ip field - supports attribute access."""
    ipv4: str


class ExchangePayload(TypedDict, total=False):
    """Payload type for Exchange operations."""
    name: str
    server_name: str
    domain_name: str
    username: str
    password: str
    ip: str
    connect_protocol: Literal["rpc-over-tcp", "rpc-over-http", "rpc-over-https"]
    validate_server_certificate: Literal["disable", "enable"]
    auth_type: Literal["spnego", "ntlm", "kerberos"]
    auth_level: Literal["connect", "call", "packet", "integrity", "privacy"]
    http_auth_type: Literal["basic", "ntlm"]
    ssl_min_proto_version: Literal["default", "SSLv3", "TLSv1", "TLSv1-1", "TLSv1-2", "TLSv1-3"]
    auto_discover_kdc: Literal["enable", "disable"]
    kdc_ip: str | list[str] | list[dict[str, Any]] | list[ExchangeKdcipItem]


# ================================================================
# Response Types (TypedDict for dict-style access)
# ================================================================

class ExchangeResponse(TypedDict, total=False):
    """Response type for Exchange - use with .dict property for typed dict access."""
    name: str
    server_name: str
    domain_name: str
    username: str
    password: str
    ip: str
    connect_protocol: Literal["rpc-over-tcp", "rpc-over-http", "rpc-over-https"]
    validate_server_certificate: Literal["disable", "enable"]
    auth_type: Literal["spnego", "ntlm", "kerberos"]
    auth_level: Literal["connect", "call", "packet", "integrity", "privacy"]
    http_auth_type: Literal["basic", "ntlm"]
    ssl_min_proto_version: Literal["default", "SSLv3", "TLSv1", "TLSv1-1", "TLSv1-2", "TLSv1-3"]
    auto_discover_kdc: Literal["enable", "disable"]
    kdc_ip: list[ExchangeKdcipItem]


# ================================================================
# Response Types (Class for attribute access)
# ================================================================


class ExchangeObject(FortiObject):
    """Typed FortiObject for Exchange with field access."""
    name: str
    server_name: str
    domain_name: str
    username: str
    password: str
    ip: str
    connect_protocol: Literal["rpc-over-tcp", "rpc-over-http", "rpc-over-https"]
    validate_server_certificate: Literal["disable", "enable"]
    auth_type: Literal["spnego", "ntlm", "kerberos"]
    auth_level: Literal["connect", "call", "packet", "integrity", "privacy"]
    http_auth_type: Literal["basic", "ntlm"]
    ssl_min_proto_version: Literal["default", "SSLv3", "TLSv1", "TLSv1-1", "TLSv1-2", "TLSv1-3"]
    auto_discover_kdc: Literal["enable", "disable"]
    kdc_ip: list[ExchangeKdcipItem]


# ================================================================
# Main Endpoint Class
# ================================================================

class Exchange:
    """
    
    Endpoint: user/exchange
    Category: cmdb
    MKey: name
    """
    
    # Class attributes for introspection
    endpoint: ClassVar[str] = ...
    path: ClassVar[str] = ...
    category: ClassVar[str] = ...
    mkey: ClassVar[str] = ...
    capabilities: ClassVar[dict[str, Any]] = ...
    
    def __init__(self, client: Any) -> None:
        """Initialize endpoint with HTTP client."""
        ...
    
    # ================================================================
    # GET Methods
    # ================================================================
    
    # CMDB with mkey - overloads for single vs list returns
    @overload
    def get(
        self,
        name: str,
        *,
        filter: str | list[str] | None = ...,
        count: int | None = ...,
        start: int | None = ...,
        payload_dict: dict[str, Any] | None = ...,
        range: list[int] | None = ...,
        sort: str | None = ...,
        format: str | None = ...,
        action: str | None = ...,
        vdom: str | bool | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> ExchangeObject: ...
    
    @overload
    def get(
        self,
        *,
        filter: str | list[str] | None = ...,
        count: int | None = ...,
        start: int | None = ...,
        payload_dict: dict[str, Any] | None = ...,
        range: list[int] | None = ...,
        sort: str | None = ...,
        format: str | None = ...,
        action: str | None = ...,
        vdom: str | bool | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> FortiObjectList[ExchangeObject]: ...
    
    def get_schema(
        self,
        vdom: str | None = ...,
        format: str = ...,
    ) -> FortiObject: ...

    # ================================================================
    # POST Method
    # ================================================================
    
    def post(
        self,
        payload_dict: ExchangePayload | None = ...,
        name: str | None = ...,
        server_name: str | None = ...,
        domain_name: str | None = ...,
        username: str | None = ...,
        password: str | None = ...,
        ip: str | None = ...,
        connect_protocol: Literal["rpc-over-tcp", "rpc-over-http", "rpc-over-https"] | None = ...,
        validate_server_certificate: Literal["disable", "enable"] | None = ...,
        auth_type: Literal["spnego", "ntlm", "kerberos"] | None = ...,
        auth_level: Literal["connect", "call", "packet", "integrity", "privacy"] | None = ...,
        http_auth_type: Literal["basic", "ntlm"] | None = ...,
        ssl_min_proto_version: Literal["default", "SSLv3", "TLSv1", "TLSv1-1", "TLSv1-2", "TLSv1-3"] | None = ...,
        auto_discover_kdc: Literal["enable", "disable"] | None = ...,
        kdc_ip: str | list[str] | list[dict[str, Any]] | list[ExchangeKdcipItem] | None = ...,
        vdom: str | bool | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> ExchangeObject: ...

    # ================================================================
    # PUT Method
    # ================================================================
    
    def put(
        self,
        payload_dict: ExchangePayload | None = ...,
        name: str | None = ...,
        server_name: str | None = ...,
        domain_name: str | None = ...,
        username: str | None = ...,
        password: str | None = ...,
        ip: str | None = ...,
        connect_protocol: Literal["rpc-over-tcp", "rpc-over-http", "rpc-over-https"] | None = ...,
        validate_server_certificate: Literal["disable", "enable"] | None = ...,
        auth_type: Literal["spnego", "ntlm", "kerberos"] | None = ...,
        auth_level: Literal["connect", "call", "packet", "integrity", "privacy"] | None = ...,
        http_auth_type: Literal["basic", "ntlm"] | None = ...,
        ssl_min_proto_version: Literal["default", "SSLv3", "TLSv1", "TLSv1-1", "TLSv1-2", "TLSv1-3"] | None = ...,
        auto_discover_kdc: Literal["enable", "disable"] | None = ...,
        kdc_ip: str | list[str] | list[dict[str, Any]] | list[ExchangeKdcipItem] | None = ...,
        vdom: str | bool | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> ExchangeObject: ...

    # ================================================================
    # DELETE Method
    # ================================================================
    
    def delete(
        self,
        name: str | None = ...,
        vdom: str | bool | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> FortiObject: ...

    # ================================================================
    # Utility Methods
    # ================================================================
    
    def exists(
        self,
        name: str,
        vdom: str | bool | None = ...,
    ) -> bool: ...
    
    def set(
        self,
        payload_dict: ExchangePayload | None = ...,
        name: str | None = ...,
        server_name: str | None = ...,
        domain_name: str | None = ...,
        username: str | None = ...,
        password: str | None = ...,
        ip: str | None = ...,
        connect_protocol: Literal["rpc-over-tcp", "rpc-over-http", "rpc-over-https"] | None = ...,
        validate_server_certificate: Literal["disable", "enable"] | None = ...,
        auth_type: Literal["spnego", "ntlm", "kerberos"] | None = ...,
        auth_level: Literal["connect", "call", "packet", "integrity", "privacy"] | None = ...,
        http_auth_type: Literal["basic", "ntlm"] | None = ...,
        ssl_min_proto_version: Literal["default", "SSLv3", "TLSv1", "TLSv1-1", "TLSv1-2", "TLSv1-3"] | None = ...,
        auto_discover_kdc: Literal["enable", "disable"] | None = ...,
        kdc_ip: str | list[str] | list[dict[str, Any]] | list[ExchangeKdcipItem] | None = ...,
        vdom: str | bool | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> FortiObject: ...
    
    # Helper methods
    @staticmethod
    def help(field_name: str | None = ...) -> str: ...
    
    @staticmethod
    def fields(detailed: bool = ...) -> list[str] | list[dict[str, Any]]: ...
    
    @staticmethod
    def field_info(field_name: str) -> FortiObject: ...
    
    @staticmethod
    def validate_field(name: str, value: Any) -> bool: ...
    
    @staticmethod
    def required_fields() -> list[str]: ...
    
    @staticmethod
    def defaults() -> FortiObject: ...
    
    @staticmethod
    def schema() -> FortiObject: ...


__all__ = [
    "Exchange",
    "ExchangePayload",
    "ExchangeResponse",
    "ExchangeObject",
]


"""Type stubs for LOG category."""

from __future__ import annotations

from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from hfortix_core.http.interface import IHTTPClient
    from .custom_field import CustomField
    from .eventfilter import Eventfilter
    from .gui_display import GuiDisplay
    from .setting import Setting
    from .threat_weight import ThreatWeight
    from .disk import Disk
    from .fortianalyzer import Fortianalyzer
    from .fortianalyzer2 import Fortianalyzer2
    from .fortianalyzer3 import Fortianalyzer3
    from .fortianalyzer_cloud import FortianalyzerCloud
    from .fortiguard import Fortiguard
    from .memory import Memory
    from .null_device import NullDevice
    from .syslogd import Syslogd
    from .syslogd2 import Syslogd2
    from .syslogd3 import Syslogd3
    from .syslogd4 import Syslogd4
    from .tacacs_plusaccounting import TacacsPlusaccounting
    from .tacacs_plusaccounting2 import TacacsPlusaccounting2
    from .tacacs_plusaccounting3 import TacacsPlusaccounting3
    from .webtrends import Webtrends

__all__ = [
    "CustomField",
    "Eventfilter",
    "GuiDisplay",
    "Setting",
    "ThreatWeight",
    "Log",
]


class Log:
    """LOG API category.
    
    All endpoints return FortiObject instances with:
    - Attribute access: response.field
    - Dictionary access: response["field"]
    - Convert to dict: response.dict or response.json
    """
    
    disk: Disk
    fortianalyzer: Fortianalyzer
    fortianalyzer2: Fortianalyzer2
    fortianalyzer3: Fortianalyzer3
    fortianalyzer_cloud: FortianalyzerCloud
    fortiguard: Fortiguard
    memory: Memory
    null_device: NullDevice
    syslogd: Syslogd
    syslogd2: Syslogd2
    syslogd3: Syslogd3
    syslogd4: Syslogd4
    tacacs_plusaccounting: TacacsPlusaccounting
    tacacs_plusaccounting2: TacacsPlusaccounting2
    tacacs_plusaccounting3: TacacsPlusaccounting3
    webtrends: Webtrends
    custom_field: CustomField
    eventfilter: Eventfilter
    gui_display: GuiDisplay
    setting: Setting
    threat_weight: ThreatWeight

    def __init__(self, client: IHTTPClient, vdom: str | None = None) -> None:
        """Initialize log category with HTTP client."""
        ...



"""FortiOS CMDB - Cluster category"""

from .state import State

__all__ = [
    "Cluster",
    "State",
]


class Cluster:
    """Cluster endpoints wrapper for CMDB API."""

    def __init__(self, client):
        """Cluster endpoints.
        
        Args:
            client: HTTP client instance for API communication
        """
        self.state = State(client)



"""
Pydantic Models for CMDB - webfilter/override

Runtime validation models for webfilter/override configuration.
Generated from FortiOS schema version unknown.
"""

from __future__ import annotations

from pydantic import BaseModel, Field, field_validator
from typing import Any, Literal, Optional
from enum import Enum

# ============================================================================
# Enum Definitions (for fields with 4+ allowed values)
# ============================================================================

class OverrideScopeEnum(str, Enum):
    """Allowed values for scope field."""
    USER = "user"
    USER_GROUP = "user-group"
    IP = "ip"
    IP6 = "ip6"


# ============================================================================
# Main Model
# ============================================================================

class OverrideModel(BaseModel):
    """
    Pydantic model for webfilter/override configuration.
    
    Configure FortiGuard Web Filter administrative overrides.
    
    Validation Rules:        - id_: min=0 max=4294967295 pattern=        - status: pattern=        - scope: pattern=        - ip: pattern=        - user: max_length=64 pattern=        - user_group: max_length=63 pattern=        - old_profile: max_length=47 pattern=        - new_profile: max_length=47 pattern=        - ip6: pattern=        - expires: pattern=        - initiator: max_length=64 pattern=    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        validate_assignment = True  # Validate on attribute assignment
        use_enum_values = True  # Use enum values instead of names
    
    # ========================================================================
    # Model Fields
    # ========================================================================
    
    id_: int | None = Field(ge=0, le=4294967295, default=0, serialization_alias="id", description="Override rule ID.")    
    status: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable override rule.")    
    scope: OverrideScopeEnum | None = Field(default=OverrideScopeEnum.USER, description="Override either the specific user, user group, IPv4 address, or IPv6 address.")    
    ip: str = Field(default="0.0.0.0", description="IPv4 address which the override applies.")    
    user: str = Field(max_length=64, description="Name of the user which the override applies.")    
    user_group: str = Field(max_length=63, description="Specify the user group for which the override applies.")  # datasource: ['user.group.name']    
    old_profile: str = Field(max_length=47, description="Name of the web filter profile which the override applies.")  # datasource: ['webfilter.profile.name']    
    new_profile: str = Field(max_length=47, description="Name of the new web filter profile used by the override.")  # datasource: ['webfilter.profile.name']    
    ip6: str = Field(default="::", description="IPv6 address which the override applies.")    
    expires: str = Field(default="1970/01/01 00:00:00", description="Override expiration date and time, from 5 minutes to 365 from now (format: yyyy/mm/dd hh:mm:ss).")    
    initiator: str | None = Field(max_length=64, default=None, description="Initiating user of override (read-only setting).")    
    # ========================================================================
    # Custom Validators
    # ========================================================================
    
    @field_validator('user_group')
    @classmethod
    def validate_user_group(cls, v: Any) -> Any:
        """
        Validate user_group field.
        
        Datasource: ['user.group.name']
        
        Note:
            This validator only checks basic constraints.
            To validate that referenced object exists, query the API.
        """
        # Basic validation passed via Field() constraints
        # Additional datasource validation could be added here
        return v    
    @field_validator('old_profile')
    @classmethod
    def validate_old_profile(cls, v: Any) -> Any:
        """
        Validate old_profile field.
        
        Datasource: ['webfilter.profile.name']
        
        Note:
            This validator only checks basic constraints.
            To validate that referenced object exists, query the API.
        """
        # Basic validation passed via Field() constraints
        # Additional datasource validation could be added here
        return v    
    @field_validator('new_profile')
    @classmethod
    def validate_new_profile(cls, v: Any) -> Any:
        """
        Validate new_profile field.
        
        Datasource: ['webfilter.profile.name']
        
        Note:
            This validator only checks basic constraints.
            To validate that referenced object exists, query the API.
        """
        # Basic validation passed via Field() constraints
        # Additional datasource validation could be added here
        return v    
    # ========================================================================
    # Helper Methods
    # ========================================================================
    
    def to_fortios_dict(self) -> dict[str, Any]:
        """
        Convert model to FortiOS API payload format.
        
        Returns:
            Dict suitable for POST/PUT operations
        """
        # Export with exclude_none to avoid sending null values
        return self.model_dump(exclude_none=True, by_alias=True)
    
    @classmethod
    def from_fortios_response(cls, data: dict[str, Any]) -> "OverrideModel":
        """
        Create model instance from FortiOS API response.
        
        Args:
            data: Response data from API
            
        Returns:
            Validated model instance
        """
        return cls(**data)
    # ========================================================================
    # Datasource Validation Methods
    # ========================================================================    
    async def validate_user_group_references(self, client: Any) -> list[str]:
        """
        Validate user_group references exist in FortiGate.
        
        This method checks if referenced objects exist by calling exists() on
        the appropriate API endpoints. This is an OPTIONAL validation step that
        can be called before posting to the API to catch reference errors early.
        
        Datasource endpoints checked:
        - user/group        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of validation error messages (empty if all valid)
            
        Example:
            >>> from hfortix_fortios import FortiOS
            >>> 
            >>> fgt = FortiOS(host="192.168.1.1", token="your-token")
            >>> policy = OverrideModel(
            ...     user_group="invalid-name",
            ... )
            >>> 
            >>> # Validate before posting
            >>> errors = await policy.validate_user_group_references(fgt._client)
            >>> if errors:
            ...     print("Validation failed:", errors)
            ... else:
            ...     result = await fgt.api.cmdb.webfilter.override.post(policy.to_fortios_dict())
        """
        errors: list[str] = []
        
        # Validate scalar field
        value = getattr(self, "user_group", None)
        if not value:
            return errors
        
        # Check all datasource endpoints
        found = False
        if await client.api.cmdb.user.group.exists(value):
            found = True
        
        if not found:
            errors.append(
                f"User-Group '{value}' not found in "
                "user/group"
            )        
        return errors    
    async def validate_old_profile_references(self, client: Any) -> list[str]:
        """
        Validate old_profile references exist in FortiGate.
        
        This method checks if referenced objects exist by calling exists() on
        the appropriate API endpoints. This is an OPTIONAL validation step that
        can be called before posting to the API to catch reference errors early.
        
        Datasource endpoints checked:
        - webfilter/profile        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of validation error messages (empty if all valid)
            
        Example:
            >>> from hfortix_fortios import FortiOS
            >>> 
            >>> fgt = FortiOS(host="192.168.1.1", token="your-token")
            >>> policy = OverrideModel(
            ...     old_profile="invalid-name",
            ... )
            >>> 
            >>> # Validate before posting
            >>> errors = await policy.validate_old_profile_references(fgt._client)
            >>> if errors:
            ...     print("Validation failed:", errors)
            ... else:
            ...     result = await fgt.api.cmdb.webfilter.override.post(policy.to_fortios_dict())
        """
        errors: list[str] = []
        
        # Validate scalar field
        value = getattr(self, "old_profile", None)
        if not value:
            return errors
        
        # Check all datasource endpoints
        found = False
        if await client.api.cmdb.webfilter.profile.exists(value):
            found = True
        
        if not found:
            errors.append(
                f"Old-Profile '{value}' not found in "
                "webfilter/profile"
            )        
        return errors    
    async def validate_new_profile_references(self, client: Any) -> list[str]:
        """
        Validate new_profile references exist in FortiGate.
        
        This method checks if referenced objects exist by calling exists() on
        the appropriate API endpoints. This is an OPTIONAL validation step that
        can be called before posting to the API to catch reference errors early.
        
        Datasource endpoints checked:
        - webfilter/profile        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of validation error messages (empty if all valid)
            
        Example:
            >>> from hfortix_fortios import FortiOS
            >>> 
            >>> fgt = FortiOS(host="192.168.1.1", token="your-token")
            >>> policy = OverrideModel(
            ...     new_profile="invalid-name",
            ... )
            >>> 
            >>> # Validate before posting
            >>> errors = await policy.validate_new_profile_references(fgt._client)
            >>> if errors:
            ...     print("Validation failed:", errors)
            ... else:
            ...     result = await fgt.api.cmdb.webfilter.override.post(policy.to_fortios_dict())
        """
        errors: list[str] = []
        
        # Validate scalar field
        value = getattr(self, "new_profile", None)
        if not value:
            return errors
        
        # Check all datasource endpoints
        found = False
        if await client.api.cmdb.webfilter.profile.exists(value):
            found = True
        
        if not found:
            errors.append(
                f"New-Profile '{value}' not found in "
                "webfilter/profile"
            )        
        return errors    
    async def validate_all_references(self, client: Any) -> list[str]:
        """
        Validate ALL datasource references in this model.
        
        Convenience method that runs all validate_*_references() methods
        and aggregates the results.
        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of all validation errors found
            
        Example:
            >>> errors = await policy.validate_all_references(fgt._client)
            >>> if errors:
            ...     for error in errors:
            ...         print(f"  - {error}")
        """
        all_errors = []
        
        errors = await self.validate_user_group_references(client)
        all_errors.extend(errors)        
        errors = await self.validate_old_profile_references(client)
        all_errors.extend(errors)        
        errors = await self.validate_new_profile_references(client)
        all_errors.extend(errors)        
        return all_errors

# ============================================================================
# Type Aliases for Convenience
# ============================================================================

Dict = dict[str, Any]  # For backward compatibility

# ============================================================================
# Module Exports
# ============================================================================

__all__ = [
    "OverrideModel",]


# ============================================================================
# Generated by hfortix generator v0.6.0
# Schema: 1.7.0
# Generated: 2026-01-17T17:56:11.069455Z
# ============================================================================


"""FortiOS CMDB - Lookup category (stub)"""

from typing import Any
from ..lookup_base import Lookup as LookupBase
from .ha_peer import HaPeer

class Lookup(LookupBase):
    """Lookup endpoints wrapper for CMDB API."""

    ha_peer: HaPeer

    def __init__(self, client: Any) -> None:
        """Initialize endpoint with HTTP client."""
        ...



"""Type stubs for ENDPOINT_CONTROL category."""

from __future__ import annotations

from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from hfortix_core.http.interface import IHTTPClient
    from .fctems import Fctems
    from .fctems_override import FctemsOverride
    from .settings import Settings

__all__ = [
    "Fctems",
    "FctemsOverride",
    "Settings",
    "EndpointControl",
]


class EndpointControl:
    """ENDPOINT_CONTROL API category.
    
    All endpoints return FortiObject instances with:
    - Attribute access: response.field
    - Dictionary access: response["field"]
    - Convert to dict: response.dict or response.json
    """
    
    fctems: Fctems
    fctems_override: FctemsOverride
    settings: Settings

    def __init__(self, client: IHTTPClient, vdom: str | None = None) -> None:
        """Initialize endpoint_control category with HTTP client."""
        ...



"""
Pydantic Models for CMDB - certificate/remote

Runtime validation models for certificate/remote configuration.
Generated from FortiOS schema version unknown.
"""

from __future__ import annotations

from pydantic import BaseModel, Field
from typing import Any, Literal, Optional

# ============================================================================
# Enum Definitions (for fields with 4+ allowed values)
# ============================================================================


# ============================================================================
# Main Model
# ============================================================================

class RemoteModel(BaseModel):
    """
    Pydantic model for certificate/remote configuration.
    
    Remote certificate as a PEM file.
    
    Validation Rules:        - name: max_length=35 pattern=        - remote: pattern=        - range_: pattern=        - source: pattern=    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        validate_assignment = True  # Validate on attribute assignment
        use_enum_values = True  # Use enum values instead of names
    
    # ========================================================================
    # Model Fields
    # ========================================================================
    
    name: str = Field(max_length=35, description="Name.")    
    remote: str | None = Field(default=None, description="Remote certificate.")    
    range_: Literal["global", "vdom"] | None = Field(default="global", serialization_alias="range", description="Either the global or VDOM IP address range for the remote certificate.")    
    source: Literal["factory", "user", "bundle"] | None = Field(default="user", description="Remote certificate source type.")    
    # ========================================================================
    # Custom Validators
    # ========================================================================
    
    # ========================================================================
    # Helper Methods
    # ========================================================================
    
    def to_fortios_dict(self) -> dict[str, Any]:
        """
        Convert model to FortiOS API payload format.
        
        Returns:
            Dict suitable for POST/PUT operations
        """
        # Export with exclude_none to avoid sending null values
        return self.model_dump(exclude_none=True, by_alias=True)
    
    @classmethod
    def from_fortios_response(cls, data: dict[str, Any]) -> "RemoteModel":
        """
        Create model instance from FortiOS API response.
        
        Args:
            data: Response data from API
            
        Returns:
            Validated model instance
        """
        return cls(**data)

# ============================================================================
# Type Aliases for Convenience
# ============================================================================

Dict = dict[str, Any]  # For backward compatibility

# ============================================================================
# Module Exports
# ============================================================================

__all__ = [
    "RemoteModel",]


# ============================================================================
# Generated by hfortix generator v0.6.0
# Schema: 1.7.0
# Generated: 2026-01-17T17:56:13.884247Z
# ============================================================================